#!/opt/rh/rh-ruby23/root/usr/bin/ruby
# -*- coding: utf-8 -*-

class CloudConfigUpdateManager

  private

  def initialize(argc = 'cloud_config_update', argv = [])
    @argc = argc.downcase.to_sym
    @argv = argv
    run
  end

  def usage
    STDERR.puts "usage:"
    STDERR.puts " #{@argc}"
    STDERR.puts "usage(show):"
    STDERR.puts " #{@argc} show"
    STDERR.puts "usage(update):"
    STDERR.puts " #{@argc} update"
    exit 1
  end

  def db(params = {})
    if block_given? then
      begin
        yield(params)
      rescue Exception => e
        STDERR.puts e.message
        if e.backtrace then
          e.backtrace.each do |i|
            STDERR.puts i
          end
        end
        exit 1
      end
    end
  end

  def run
    usage if @argv.size == 0
    case @argv[0].downcase.to_sym
    when :show then run_show
    when :update then run_update
    else usage
    end
  end

  def run_show
    usage if @argv.size != 1
    ret = db do
      ret = 1
      # メール通知設定用のURL
      puts "メール通知設定用のURL:"
      puts "  現設定: #{Setting.protocol}://#{Setting.host_name}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_URL'] and ENV['SIDFM_VMCLOUD_URL'].size > 0) ? ENV['SIDFM_VMCLOUD_URL'] : '(none)'}"
      puts ""
      # メール通知設定用のメールアドレス
      puts "メール通知設定用のメールアドレス:"
      puts "  現設定: #{Setting.mail_from}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_MAIL_FROM'] and ENV['SIDFM_VMCLOUD_MAIL_FROM'].size > 0) ? ENV['SIDFM_VMCLOUD_MAIL_FROM'] : '(none)'}"
      puts ""
      # 脆弱性同期時に一度に同期処理をするホスト数
      puts "脆弱性同期時に一度に同期処理をするホスト数:"
      puts "  現設定: #{Sidfm.parallel_hosts}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_PARALLEL_HOSTS'] and ENV['SIDFM_VMCLOUD_PARALLEL_HOSTS'].size > 0) ? ENV['SIDFM_VMCLOUD_PARALLEL_HOSTS'] : '(none)'}"
      puts ""
      # 脆弱性同期の周期
      puts "脆弱性同期の周期:"
      puts "  現設定: #{Sidfm.vulsync_interval}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_VULSYNC_INTERVAL'] and ENV['SIDFM_VMCLOUD_VULSYNC_INTERVAL'].size > 0) ? ENV['SIDFM_VMCLOUD_VULSYNC_INTERVAL'] : '(none)'}"
      puts ""
      # ルールの最大登録件数
      puts "ルールの最大登録件数:"
      puts "  現設定: #{Sidfm.max_rules}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_MAX_RULES'] and ENV['SIDFM_VMCLOUD_MAX_RULES'].size > 0) ? ENV['SIDFM_VMCLOUD_MAX_RULES'] : '(none)'} "
      puts ""
      # CVE一覧画面の最大ダウンロード件数
      puts "CVE一覧画面の最大ダウンロード件数:"
      puts "  現設定: #{Sidfm.get_cve_max_download_size}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_CVE_MAX_DOWNLOAD_SIZE'] and ENV['SIDFM_VMCLOUD_CVE_MAX_DOWNLOAD_SIZE'].size > 0) ? ENV['SIDFM_VMCLOUD_CVE_MAX_DOWNLOAD_SIZE'] : '(none)'}"
      puts ""
      # 履歴情報画面の最大ダウンロード件数
      puts "履歴情報画面の最大ダウンロード件数:"
      puts "  現設定: #{Sidfm.get_history_max_download_size}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_HISTORY_MAX_DOWNLOAD_SIZE'] and ENV['SIDFM_VMCLOUD_HISTORY_MAX_DOWNLOAD_SIZE'].size > 0) ? ENV['SIDFM_VMCLOUD_HISTORY_MAX_DOWNLOAD_SIZE'] : '(none)'}"
      puts ""
      # 非同期日数設定
      puts "非同期日数設定:"
      puts "  現設定: #{Sidfm.syncing_check_span}"
      puts "  環境変数: #{(ENV['SIDFM_VMCLOUD_SYNCING_CHECK_SPAN'] and ENV['SIDFM_VMCLOUD_SYNCING_CHECK_SPAN'].size > 0) ? ENV['SIDFM_VMCLOUD_SYNCING_CHECK_SPAN'] : '(none)'}"
      ret = 0
      ret
    end
    exit ret
  end

  def run_update
    usage if @argv.size != 1
    ret = db do
      ret = 1
      # メール通知設定用のURL
      puts "メール通知設定用のURL:"
      puts "  現設定: #{Setting.protocol}://#{Setting.host_name}"
      vmcloud_url = (ENV['SIDFM_VMCLOUD_URL'] and ENV['SIDFM_VMCLOUD_URL'].size > 0) ? ENV['SIDFM_VMCLOUD_URL'] : nil
      puts "  環境変数: #{vmcloud_url || '(none)'}"
      if vmcloud_url then
        url = URI.parse(vmcloud_url)
        if url then
          if 'http' != url.scheme and 'https' != url.scheme then
            raise Exception, "Invalid Scheme. scheme=#{url.scheme}"
          end
          if !url.host or url.host.size == 0 then
            raise Exception, "Invalid Hostname. hostnmae=#{url.host}"
          end
          if !url.port or url.port.to_i == 0 or url.port.to_i > 65535 then
            raise Exception, "Invalid Port number. port=#{url.port}"
          end
          if !url.path or url.path.size == 0 then
            raise Exception, "Invalid Path. path=#{url.path}"
          end
          Setting.transaction do
            protocol = Setting.lock.where(:name => 'protocol').first
            host_name = Setting.lock.where(:name => 'host_name').first
            new_host_name = ''
            new_host_name << url.host
            if !('http' == url.scheme and 80 == url.port.to_i) and
                !('https' == url.scheme and 443 == url.port.to_i) then
              new_host_name << ":#{url.port}"
            end
            if url.path[0] != '/' then
              new_host_name << '/'
            end
            new_host_name << url.path
            if protocol then
              protocol.value = url.scheme
              protocol.save
            else
              protocol = Setting.create(:name => 'procotol', :value => url.scheme)
            end
            if host_name then
              host_name.value = new_host_name
              host_name.save
            else
              host_name = Setting.create(:name => 'host_name', :value => new_host_name)
            end
            puts "  設定後: #{protocol.value}://#{host_name.value}"
          end
        else
          raise Exception, 'invalid url: #{vmcloud_url}'
        end
      else
        puts "  変更なし"
      end
      puts ""
      # メール通知設定用のメールアドレス
      puts "メール通知設定用のメールアドレス:"
      puts "  現設定: #{Setting.mail_from}"
      vmcloud_mail_from = (ENV['SIDFM_VMCLOUD_MAIL_FROM'] and ENV['SIDFM_VMCLOUD_MAIL_FROM'].size > 0) ? ENV['SIDFM_VMCLOUD_MAIL_FROM'] : nil
      puts "  環境変数: #{vmcloud_mail_from || '(none)'}"
      if vmcloud_mail_from then
        Setting.transaction do
          mail_from = Setting.lock.where(:name => 'mail_from').first
          if mail_from then
            mail_from.value = vmcloud_mail_from
            mail_from.save
          else
            mail_from = Setting.create(:name => 'mail_from', :value => vmcloud_mail_from)
          end
          puts "  設定後: #{mail_from.value}"
        end
      else
        puts "  変更なし"
      end
      puts ""
      # 脆弱性同期時に一度に同期処理をするホスト数
      puts "脆弱性同期時に一度に同期処理をするホスト数:"
      puts "  現設定: #{Sidfm.parallel_hosts}"
      vmcloud_parallel_hosts = (ENV['SIDFM_VMCLOUD_PARALLEL_HOSTS'] and ENV['SIDFM_VMCLOUD_PARALLEL_HOSTS'].size > 0) ? ENV['SIDFM_VMCLOUD_PARALLEL_HOSTS'] : nil
      puts "  環境変数: #{vmcloud_parallel_hosts || '(none)'}"
      if vmcloud_parallel_hosts then
        Sidfm.parallel_hosts = vmcloud_parallel_hosts.to_i
        puts "  設定後: #{Sidfm.parallel_hosts}"
      else
        puts "  変更なし"
      end
      puts ""
      # 脆弱性同期の周期
      puts "脆弱性同期の周期:"
      puts "  現設定: #{Sidfm.vulsync_interval}"
      vmcloud_vulsync_interval = (ENV['SIDFM_VMCLOUD_VULSYNC_INTERVAL'] and ENV['SIDFM_VMCLOUD_VULSYNC_INTERVAL'].size > 0) ? ENV['SIDFM_VMCLOUD_VULSYNC_INTERVAL'] : nil
      puts "  環境変数: #{vmcloud_vulsync_interval || '(none)'}"
      if vmcloud_vulsync_interval then
        Sidfm.vulsync_interval = vmcloud_vulsync_interval.to_i
        puts "  設定後: #{Sidfm.vulsync_interval}"
      else
        puts "  変更なし"
      end
      puts ""
      # ルールの最大登録件数
      puts "ルールの最大登録件数:"
      puts "  現設定: #{Sidfm.max_rules}"
      vmcloud_max_rules = (ENV['SIDFM_VMCLOUD_MAX_RULES'] and ENV['SIDFM_VMCLOUD_MAX_RULES'].size > 0) ? ENV['SIDFM_VMCLOUD_MAX_RULES'] : nil
      puts "  環境変数: #{vmcloud_max_rules || '(none)'} "
      if vmcloud_max_rules then
        Sidfm.max_rules = vmcloud_max_rules.to_i
        puts "  設定後: #{Sidfm.max_rules}"
      else
        puts "  変更なし"
      end
      puts ""
      # CVE一覧画面の最大ダウンロード件数
      puts "CVE一覧画面の最大ダウンロード件数:"
      puts "  現設定: #{Sidfm.get_cve_max_download_size}"
      vmcloud_cve_max_download_size = (ENV['SIDFM_VMCLOUD_CVE_MAX_DOWNLOAD_SIZE'] and ENV['SIDFM_VMCLOUD_CVE_MAX_DOWNLOAD_SIZE'].size > 0) ? ENV['SIDFM_VMCLOUD_CVE_MAX_DOWNLOAD_SIZE'] : nil
      puts "  環境変数: #{vmcloud_cve_max_download_size || '(none)'}"
      if vmcloud_cve_max_download_size then
        Sidfm.set_cve_max_download_size = vmcloud_cve_max_download_size.to_i
        puts "  設定後: #{Sidfm.get_cve_max_download_size}"
      else
        puts "  変更なし"
      end
      puts ""
      # 履歴情報画面の最大ダウンロード件数
      puts "履歴情報画面の最大ダウンロード件数:"
      puts "  現設定: #{Sidfm.get_history_max_download_size}"
      vmcloud_history_max_download_size = (ENV['SIDFM_VMCLOUD_HISTORY_MAX_DOWNLOAD_SIZE'] and ENV['SIDFM_VMCLOUD_HISTORY_MAX_DOWNLOAD_SIZE'].size > 0) ? ENV['SIDFM_VMCLOUD_HISTORY_MAX_DOWNLOAD_SIZE'] : nil
      puts "  環境変数: #{vmcloud_history_max_download_size || '(none)'}"
      if vmcloud_history_max_download_size then
        Sidfm.set_history_max_download_size = vmcloud_history_max_download_size.to_i
        puts "  設定後: #{Sidfm.get_history_max_download_size}"
      else
        puts "  変更なし"
      end
      puts ""
      # 非同期日数設定
      puts "非同期日数設定:"
      puts "  現設定: #{Sidfm.syncing_check_span}"
      vmcloud_syncing_check_span = (ENV['SIDFM_VMCLOUD_SYNCING_CHECK_SPAN'] and ENV['SIDFM_VMCLOUD_SYNCING_CHECK_SPAN'].size > 0) ? ENV['SIDFM_VMCLOUD_SYNCING_CHECK_SPAN'] : nil
      puts "  環境変数: #{vmcloud_syncing_check_span || '(none)'}"
      if vmcloud_syncing_check_span then
        Sidfm.syncing_check_span = vmcloud_syncing_check_span.to_i
        puts "  設定後: #{Sidfm.syncing_check_span}"
      else
        puts "  変更なし"
      end
      ret = 0
      ret
    end
    exit ret
  end

end

if $0 == __FILE__ then
  id = CloudConfigUpdateManager.new($0, ARGV)
end
