#!/opt/rh/rh-ruby23/root/usr/bin/ruby
# -*- coding: utf-8 -*-

require 'yaml'
require 'fileutils'

class MailManage
  CONFIG_PATH = '/var/lib/redmine/config/configuration.yml'.freeze

  private

  def usage
    STDERR.puts "usage:"
    STDERR.puts " /mail_manage show"
    STDERR.puts " /mail_manage noauth smtp.example.com 25 example.com"
    STDERR.puts " /mail_manage auth smtp.example.com 25 example.com account password"
    STDERR.puts " /mail_manage auth smtp.example.com 25 example.com account password starttls"
    exit 1
  end

  def writeback
    begin
      FileUtils.mv(CONFIG_PATH, CONFIG_PATH + '.old', :force => true)
      File.open(CONFIG_PATH, 'w') do |f|
        f.print YAML.dump(@config)
      end
      STDERR.puts 'success'
    rescue Exception => e
      STDERR.puts 'failed'
    end
  end

  def initialize(argv)
    @cmd = argv.shift
    @opts = argv

    begin
      @config = YAML.load(File.read(CONFIG_PATH))
    rescue Exception => e
      STDERR.puts "access failure: #{CONFIG_PATH}"
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    end

    unless @config['default'] then
      STDERR.puts "invalid: #{CONFIG_PATH}"
      exit 1
    end

    case @cmd
    when 'show' then
      if @config['default']['email_delivery'] then
        STDERR.puts YAML.dump(@config['default']['email_delivery'])
        exit 1
      else
        STDERR.puts 'not configured'
        exit 1
      end
    when 'noauth' then
      if @opts.nil? or @opts.size != 3 then
        usage
      end
      server = @opts.shift
      if /^[a-z0-9._-]+$/i !~ server then
        STDERR.puts "invalid smtp server: #{server}"
        exit 1
      end
      port = (@opts.shift || 25).to_i
      if port <= 0 then 
        STDERR.puts "invalid smtp port: #{port}"
        exit 1
      end
      domain = @opts.shift
      if /^[a-z0-9._-]+$/i !~ domain then
        STDERR.puts "invalid domain: #{domain}"
        exit 1
      end
      @config['default']['email_delivery'] = {
        'delivery_method' => :smtp,
        'smtp_settings' => {
          'address' => server,
          'port' => port,
          'domain' => domain,
          'enable_starttls_auto' => false,
        }
      }
      writeback
    when 'auth'
      if @opts.nil? or (@opts.size != 5 and @opts.size != 6) then
        usage
      end
      server = @opts.shift
      if /^[a-z0-9._-]+$/i !~ server then
        STDERR.puts "invalid smtp server: #{server}"
        exit 1
      end
      port = (@opts.shift || 25).to_i
      if port <= 0 then 
        STDERR.puts "invalid smtp port: #{port}"
        exit 1
      end
      domain = @opts.shift
      if /^[a-z0-9._-]+$/i !~ domain then
        STDERR.puts "invalid domain: #{domain}"
        exit 1
      end
      account = @opts.shift
      password = @opts.shift
      starttls = @opts.shift
      if starttls and starttls == 'starttls' then
        starttls = true
      else
        starttls = false
      end
      @config['default']['email_delivery'] = {
        'delivery_method' => :smtp,
        'smtp_settings' => {
          'address' => server,
          'port' => port,
          'domain' => domain,
          'authentication' => :login,
          'user_name' => account,
          'password' => password,
          'enable_starttls_auto' => starttls,
        }
      }
      writeback
    else
      usage
    end
  end
end

if $0 == __FILE__ then
  id = MailManage.new(ARGV)
end
