#!/opt/rh/rh-ruby23/root/usr/bin/ruby
# -*- coding: utf-8 -*-

class UserManager

  BASE_PATH = '/var/lib/redmine'.freeze
  CONFIG_PATH = (BASE_PATH + '/config/database.yml').freeze

  private

  def initialize(argc = 'user_manage', argv = [])
    @argc = argc.downcase.to_sym
    @argv = argv
    run
  end

  def usage
    STDERR.puts "usage(lock):"
    STDERR.puts " #{@argc} lock <login name>"
    STDERR.puts "usage(unlock):"
    STDERR.puts " #{@argc} unlock <login name>"
    STDERR.puts "usage(pwd):"
    STDERR.puts " #{@argc} password <login name> <password>"
    exit 1
  end

  def db(params = {})
    if block_given? then
      begin
        yield(params)
      rescue Exception => e
        STDERR.puts e.message
        if e.backtrace then
          e.backtrace.each do |i|
            STDERR.puts i
          end
        end
        exit 1
      end
    end
  end

  def run
    usage if @argv.size == 0
    case @argv[0].downcase.to_sym
    when :lock then run_lock
    when :unlock then run_unlock
    when :password then run_password
    else usage
    end
  end

  def run_lock
    usage if @argv.size != 2
    login = @argv[1]
    ret = db({:login => login}) do |params|
      ret = 1
      target_user = User.where(:login => params[:login]).first
      if target_user then
        target_user.update_attribute :status, User::STATUS_LOCKED
        STDERR.puts "#{params[:login]} is locked."
        ret = 0
      else
        STDERR.puts "unknown login: #{login}"
      end
      ret
    end
    exit ret
  end

  def run_unlock
    usage if @argv.size != 2
    login = @argv[1]
    ret = db({:login => login}) do |params|
      ret = 1
      target_user = User.where(:login => params[:login]).first
      if target_user then
        target_user.update_attribute :status, User::STATUS_ACTIVE
        STDERR.puts "#{params[:login]} is unlocked."
        ret = 0
      else
        STDERR.puts "unknown login: #{login}"
      end
      ret
    end
    exit ret
  end

  def run_password
    usage if @argv.size != 3
    login = @argv[1]
    pwd = @argv[2]
    ret = db({:login => login, :pwd => pwd}) do |params|
      ret = 1
      target_user = User.where(:login => params[:login]).first
      if target_user then
        target_user.password = params[:pwd]
        target_user.password_confirmation = params[:pwd]
        target_user.update_hashed_password
        if target_user.save then
          STDERR.puts "#{params[:login]}'s password is updated."
        else
          STDERR.puts "#{params[:login]}'s password is not updated. (Invalid password)"
        end
        ret = 0
      else
        STDERR.puts "unknown login: #{login}"
      end
      ret
    end
    exit ret
  end

end

if $0 == __FILE__ then
  id = UserManager.new($0, ARGV)
end
