#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmProbeApikeysHelper
    # サマリー表のチェックボックス
    def create_summary_probe_apikey_chkbox(id, checked)
      content = check_box_tag(
        nil,
        id,
        checked.include?(id),
        :class => 'apikeySelect'
      )
    end
  
    # サマリー表のキー名
    def create_summary_regkey_name(apikey)
      content = content_tag(
        :div,
        :class => "summary-apikey-name d-flex"
      ) do
        concat(
          content_tag(
            :span,
            "#{apikey.name}",
            :class => "mr-auto name",
            :data => { :toggle => "tooltip", :placement => "right", :title => "#{apikey.name}" }
          )
          .concat(
            button_tag(
              'キー名変更',
              :type => 'button',
              :class => 'ml-auto update-button btn btn-default btn-xs text-nowrap',
              :data => {
                :updateapikey => true,
                :update => 'name',
                :id => apikey.id,
                :content => { :name => apikey.name }
              },
            )
          )
        )
      end
    end
  
    # APIキー
    def create_regkey_apikey(apikey, is_enddate_passed, item_name)
      content = content_tag(
        :div,
        :class => "d-flex"
      ) do
        concat(
          content_tag(
            :span,
            "#{item_name}",
            :class => "apikey-label"
          )
        )
        .concat(
          content_tag(
            :div,
            :class => "apikey-value d-flex"
          ) do
            concat(
              if apikey.api_key then
                content_tag(
                  :div,
                  "#{apikey.api_key}",
                  :class => (is_enddate_passed ? "text-danger" : "")
                )
                .concat(
                  content_tag(
                    :i,
                    nil,
                    :class => "far fa-copy ml-auto pl-1 copy-apikey cursor-pointer",
                    :data => {
                      "clipboard-text" => "#{apikey.api_key}",
                      :toggle => "tooltip",
                      :placement => "bottom",
                      :boundary => "window",
                      :title => "コピーしました!",
                      :trigger => "manual"
                    },
                    :style => "color:#605e5e;"
                  )
                )
              else
                content_tag(
                  :span,
                  "未登録",
                  :class => "slanted-font#{is_enddate_passed ? ' text-danger' : ''}"
                )
              end
            )
          end
        )
      end
    end
  
    # サマリー表のAPIキー
    def create_summary_regkey_apikey(apikey, is_enddate_passed, item_name)
      content = content_tag(
        :div,
        :class => "summary-apikey apikey-block apikey-item"
      ) do
        concat(
          content_tag(
            :div,
            :class => "apikey"
          ) do
            concat(
              create_regkey_apikey(apikey, is_enddate_passed, item_name)
            )
            .concat(
              content_tag(
                :div,
                :class => "ml-auto d-flex"
              ) do
                concat(
                  button_tag(
                    (apikey.api_key ? 'APIキー更新' : 'APIキー登録'),
                    :type => 'button',
                    :class => "update-button btn btn-default btn-xs text-nowrap exec-update-apikey-key",
                    :data => {
                      :confirmtext => "「#{apikey.name}」のAPIキーを#{apikey.api_key ? '更新' : '登録'}します",
                      :id => apikey.id,
                      :updateapikey => true,
                      :update => 'key',
                    },
                  )
                )
                .concat(
                  if apikey.api_key then
                    button_tag(
                      'APIキー削除',
                      :type => 'button',
                      :class => 'ml-1 update-button btn btn-danger btn-xs text-nowrap exec-update-apikey-key',
                      :data => {
                        :confirmtext => "「#{apikey.name}」のAPIキーを削除します", 
                        :id => apikey.id,
                        :updateapikey => true,
                        :update => 'key',
                        :removekey => true,
                      },
                    )
                  else
                    ""
                  end
                )
              end
            )
          end
        )
      end
    end
  
    def create_summary_probe_apikey_category(apikey, item_name, sidfm_roles)
      content = content_tag(
        :div,
        :class => "apikey-item d-flex"
      ) do
        concat(
          content_tag(
            :span, 
            :class => "apikey-label"
          ) do
            concat(
              content_tag(:span, "#{item_name}")
            )
            .concat(
              content_tag(
                :a,
                :data => {
                  :toggle => "tooltip",
                  :placement => "right",
                  :html => "true",
                  :title => "APIキーのカテゴリ<br>このカテゴリの権限で探索APIへのアクセスが制限されます" 
                }
              ) do
                concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
              end
            )
          end
        )
        .concat(
          content_tag(
            :div,
            apikey.category_name,
            :class => 'apikey-categoryname apikey-value',
            :data => {
              :toggle => 'tooltip',
              :title => "カテゴリid: #{apikey.sidfmd_category_id}<br>カテゴリ名: #{apikey.category_name}",
              :html => true,
              :boundary => 'window',
              :placement => 'right'
            }
          )
        )
        .concat(
          sidfm_roles.include?(:host_admin) ?
          content_tag(
            :span,
            :class => "ml-auto"
          ) do
            concat(
              button_tag(
                "#{item_name}変更",
                :type => 'button',
                :class => 'update-button btn btn-default btn-xs text-nowrap',
                :data => {
                  :updateapikey => true,
                  :update => 'category',
                  :id => apikey.id,
                  :content => {
                    :category => {
                      :category_id => apikey.sidfmd_category_id,
                      :categoryname => apikey.category_name,
                    }
                  }.to_json
                },
              )
            )
          end :
          ''
        )
      end
    end
  
    def create_summary_vm_apikey_period(apikey, is_enddate_passed, item_name)
      begin_date = apikey.begin_date ? SidfmdVmApikey.date_to_string(apikey.begin_date) : nil
      end_date = apikey.end_date ? SidfmdVmApikey.date_to_string(apikey.end_date) : nil
  
      content = content_tag(
        :div,
        :class => "apikey-item d-flex"
      ) do
        concat(
          content_tag(
            :span, 
            :class => "apikey-label"
          ) do
            concat(
              content_tag(:span, "#{item_name}")
            )
            .concat(
              content_tag(
                :a,
                :data => {
                  :toggle => "tooltip",
                  :placement => "right",
                  :html => "true",
                  :title => "APIキーの有効期限"
                }
              ) do
                concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70 ml-1px") )
              end
            )
          end
        )
        .concat(
          content_tag(
            :span,
            :class => "apikey-value"
          ) do
            concat(
              if begin_date and end_date then
                content_tag(
                  :span,
                  "#{begin_date == end_date ? "#{begin_date} のみ有効" : "#{begin_date} 〜 #{end_date}"}",
                  :class => (is_enddate_passed ? "text-danger" : "")
                )
              elsif begin_date then
                content_tag(:span, "#{begin_date} 以降有効")
              elsif end_date then
                content_tag(:span, "#{end_date} まで有効", :class => (is_enddate_passed ? "text-danger" : ""))
              else
                content_tag(:span, "無期限")
              end
            )
          end
        )
        .concat(
          content_tag(
            :span,
            :class => "ml-auto"
          ) do
            concat(
              button_tag(
                "#{item_name}変更",
                :type => 'button',
                :class => 'update-button btn btn-default btn-xs text-nowrap',
                :data => {
                  :updateapikey => true,
                  :update => 'period',
                  :id => apikey.id,
                  :content => { :begin_date => begin_date, :end_date => end_date }.to_json
                },
              )
            )
          end
        )
      end
    end
  
    def create_summary_probe_apikey_content_row(apikey, is_enddate_passed, item_names, sidfm_roles)
      content = content_tag(
       :div,
       :class => 'apikey-content-row three-items',
      ) do
        concat(create_summary_probe_apikey_category(apikey, item_names[:category], sidfm_roles))
        .concat(create_summary_vm_apikey_period(apikey, is_enddate_passed, item_names[:period]))
      end
    end 
  
    def create_summary_probe_apikey_content(apikey, item_names, sidfm_roles)
      is_enddate_passed = (apikey.end_date and apikey.end_date < Date.today) #true/falseがタイミングによって変わる可能性が0ではないので、ここで固定する
      content = content_tag(
       :div,
      ) do
        concat(
          create_summary_regkey_name(apikey)
        )
        .concat(
          content_tag(
            :div,
            :class => "summary-apikey-content"
          ) do
            concat(create_summary_regkey_apikey(apikey, is_enddate_passed, item_names[:apikey]))
            .concat(create_summary_probe_apikey_content_row(apikey, is_enddate_passed, item_names, sidfm_roles))
          end
        )
      end
    end
  
    # サマリー表の操作
    def create_summary_probe_apikey_operation(apikey)
      button_tag(
        '削除',
        :type => 'submit',
        :class => 'btn btn-xs btn-danger delete text-nowrap',
        :data => {
          :delete => 'apikey',
          :confirmtext => "「#{apikey.name}」を削除してもよろしいですか?",
          :id => apikey.id
        }
      )
    end
  
  end