# -*- coding: utf-8 -*-

class SidfmdProbe < ActiveRecord::Base
  belongs_to :sidfmd_probe_apikey
  has_one :sidfmd_probe_ssh, dependent: :destroy

  # 探索済ホストのデータのexpire処理を行う。
  # ルールは以下の通り。
  # 1. IPアドレス毎に最新のデータは削除しない
  # 2. Sidfm.probe_elapsed_days + Sidfm::PROBE_ADDITIONAL_KEEP_DAYS 日以上経過したデータは削除する
  def self.expire(probe_apikey_id, probe_key)
    datekey = SidfmdProbeSsh.
      joins(:sidfmd_probe).
      joins(:sidfmd_probe_ssh_ips).
      where(sidfmd_probes: { flg_enable: true }).
      where(sidfmd_probes: { sidfmd_probe_apikey_id: probe_apikey_id }).
      where(sidfmd_probe_ssh_ips: { ipaddress: probe_key }).
      order(:datekey => :desc).
      limit(1).
      first&.
      datekey
    ssh_ips = SidfmdProbeSshIp.
      joins(:sidfmd_probe_ssh).
      joins(:sidfmd_probe_ssh => :sidfmd_probe).
      where(sidfmd_probes: { flg_enable: true }).
      where(sidfmd_probes: { sidfmd_probe_apikey_id: probe_apikey_id }).
      where(ipaddress: probe_key).
      where.not(sidfmd_probe_sshes: { datekey: datekey }).
      where("sidfmd_probes.created_at < ?", 
            Time.current - (Sidfm.probe_elapsed_days + Sidfm::PROBE_ADDITIONAL_KEEP_DAYS))
    ssh_ip_ids = ssh_ips.pluck(:id)
    SidfmdProbeSshIpNmap.where(sidfmd_probe_ssh_ip_id: ssh_ip_ids).delete_all
    SidfmdProbeSshIpSsh.where(sidfmd_probe_ssh_ip_id: ssh_ip_ids).delete_all
    SidfmdProbeSshIpPkg.where(sidfmd_probe_ssh_ip_id: ssh_ip_ids).delete_all
    ssh_ips.delete_all
  end

  def detail
    'ssh' == self.kind ? self.sidfmd_probe_ssh : nil
  end

end
