#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdProbeApikey < ActiveRecord::Base
  belongs_to :sidfmd_category
  has_many :sidfmd_probes, dependent: :destroy
  has_many :sidfmd_hosts, dependent: :nullify
   
    NAME_LABEL = 'APIキー名'
    APIKEY_LABEL = 'APIキー'
    PERIOD_LABEL = '有効期間'
    CATEGORY_LABEL = 'カテゴリ'

    # api_keyを生成する
    # 生成したapi_keyを返す
    # SidfmdRegkey#generate_apikeyと同じ
    def generate_apikey
      # 重複回避
      salt = Sidfm.salt
      flg = true
      api_key = nil
      while flg do
        api_key = SofTekUtils.hexdigest(post: salt)
        unless self.class.find_by_api_key(api_key) then
          self.update_attribute :api_key, api_key
          flg = false
        end
      end
      api_key
    end
  
    # APIキーの登録
    def self.create_apikey(name, sidfmd_category_id)
      self.create(:name => name, :sidfmd_category_id => sidfmd_category_id)
    end
  
    # APIキーのキーを削除
    def remove_apikey
      self.update_attribute :api_key, nil
    end
  
    # キー名の更新
    def update_name(name)
      self.update_attribute :name, name
    end
  
    # 有効期間の更新
    def update_period(begin_date, end_date)
      self.update_attributes :begin_date => begin_date, :end_date => end_date
    end
  
    # カテゴリの更新
    def update_category(sidfmd_category_id)
      self.update_attribute :sidfmd_category_id, sidfmd_category_id
    end
  
    # APIキーの削除
    def self.delete_apikey(id)
      self.transaction do
        apikey = self.lock.find_by_id(id.to_i)
        apikey.destroy if apikey
      end
    end
  
    # APIキーの一括削除
    def self.bulk_delete_apikey(ids)
      self.transaction do
        self.where(:id => ids).destroy_all
      end
    end
  
    # 指定したカテゴリのAPIキーを削除
    def self.delete_category_apikeys(sidfmd_category_ids)
      self.where(:sidfmd_category_id => sidfmd_category_ids).destroy_all
    end
  
    # 日付を文字列に変換
    # UI等で、日付をここの表示方法で統一する
    def self.date_to_string(date)
      date.strftime('%Y/%m/%d')
    end
  
    # ユーザがアクセス・操作可能が可能かどうかを返す
    def is_available_user?(user_id:, user_category_ids:, sidfm_roles:)
      ret = false
      if sidfm_roles.include?(:host_admin) then
        if sidfm_roles.include?(:admin) then
          ret = true
        else
          ret = SidfmdCategoriesUser.where(:sidfmd_category_id => user_category_ids)
            .where(:user_id => user_id)
            .exists?
        end
      end
      ret
    end
  
    # APIキーで認証を行う。
    # 認証OKならSidfmdProbeApikeyのインスタンス、認証NGならnilを返す。
    def self.auth(apikey)
      return nil unless apikey
      today = Date.current
      ret = self
        .where(:api_key => apikey)
        .where("begin_date IS NULL OR begin_date <= ?", today)
        .where("end_date IS NULL OR end_date >= ?", today)
      ret.first
    end
  
  end
