#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdQueueApp < ActiveRecord::Base
  belongs_to :sidfmd_queue

  FILTER_KEYS = ["name", "version", "path", "created_at", "updated_at"].map(&:freeze).freeze

  PRODUCT_NAMES = {
    :struts => 'Apache Struts'.freeze,
    :tomcat => 'Tomcat'.freeze,
    :php => 'PHP'.freeze,
    :ruby => 'Ruby'.freeze,
    :oracle_jdk => 'Oracle Java SE JDK'.freeze,
    :oracle_sdk => 'Oracle Java SE SDK'.freeze,
    :oracle_jre => 'Oracle Java SE JRE'.freeze,
  }.freeze

  private

  # Apache Struts
  def check_application_struts(ret)
    product = SidfmdProduct.find_by_name(PRODUCT_NAMES[:struts])
    if product then
      version = product.sidfmd_versions.find_by_version(ret['version'])
      if version then
        ret['version_id'] = version[:vid]
        ret['product_id'] = product[:pid]
      else
        ret['product_id'] = product[:pid]
      end
    end
  end

  # Apache Tomcat
  def check_application_tomcat(ret)
    product = SidfmdProduct.find_by_name(PRODUCT_NAMES[:tomcat])
    if product then
      version = product.sidfmd_versions.find_by_version(ret['version'])
      if version then
        ret['version_id'] = version[:vid]
        ret['product_id'] = product[:pid]
      else
        ret['product_id'] = product[:pid]
      end
    end
  end
  
  # PHP
  def check_application_php(ret)
    product = SidfmdProduct.find_by_name(PRODUCT_NAMES[:php])
    if product then
      version = product.sidfmd_versions.find_by_version(ret['version'])
      if version then
        ret[:version_id] = version.vid
        ret[:product_id] = product.pid
      else
        ret[:product_id] = product.pid
      end
    end
  end

  # Ruby
  def check_application_ruby(ret)
    product = SidfmdProduct.find_by_name(PRODUCT_NAMES[:ruby])
    if product then
      version = product.sidfmd_versions.find_by_version(ret['version'])
      unless version then
        # 2.5.1p57 のようにパッチレベルがつくので、それを除去
        if /(\d+\.\d+\.\d+)/ =~ ret['version'] then
          version = product.sidfmd_versions.find_by_version(Regexp.last_match(1))
        end
      end
      if version then
        ret[:version_id] = version.vid
        ret[:product_id] = product.pid
      else
        ret[:product_ids] = product.pid
      end
    end
  end

  # Oracke JDK
  def check_application_jdk(ret)
    product_jdk = SidfmdProduct.find_by_name(PRODUCT_NAMES[:oracle_jdk])
    product_sdk = SidfmdProduct.find_by_name(PRODUCT_NAMES[:oracle_sdk])
    version = nil
    if /jdk/i =~ ret['path'] and /jre/i !~ ret['path'] then 
      if product_jdk then
        version = product_jdk.sidfmd_versions.find_by_version(ret['version'])
      end
      unless version then
        if product_sdk then
          version = product_sdk.sidfmd_versions.find_by_version(ret['version'])
        end
      end
    end
    if version then
      ret[:version_id] = version[:vid]
      if product_jdk then
        ret[:product_id] = product_jdk[:pid]
      elsif product_sdk then
        ret[:product_id] = product_sdk[:pid]
      end
    else
      ret[:product_id] = product_jdk[:pid]
    end
  end

  # Oracle JRE
  def check_application_jre(ret)
    product_jre = SidfmdProduct.find_by_name(PRODUCT_NAMES[:oracle_jre])
    product_sdk = SidfmdProduct.find_by_name(PRODUCT_NAMES[:oracle_sdk])
    version = nil
    if /jre/ =~ ret['path'] then
      if product_jre then
        version = product_jre.sidfmd_versions.find_by_version(ret['version'])
      end
      unless version then
        if product_sdk then
          version = product_sdk.sidfmd_versions.find_by_version(ret['version'])
        end
      end
    end
    if version then
      ret[:version_id] = version[:vid]
      if product_jre
        ret[:product_id] = product_jre[:pid]
      elsif product_sdk then
        ret[:product_id] = product_sdk[:pid]
      end
    else
      if product_jre
        ret[:product_id] = product_jre[:pid]
      elsif product_sdk then
        ret[:product_id] = product_sdk[:pid]
      end
    end
  end

  public

  # VM Agentで取得したアプリケーションとの照合
  def get_agent_product
    ret = self.attributes.select {|k, v| FILTER_KEYS.include?(k)}
    case self[:name]
    when 'Apache Struts' then check_application_struts(ret)
    when 'Apache Tomcat' then check_application_tomcat(ret)
    when 'PHP' then check_application_php(ret)
    when 'ruby' then check_application_ruby(ret)
    when 'Oracle Java' then check_application_jdk(ret)
    end
    ret
  end
end
