#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdQueueWin < ActiveRecord::Base
  self.primary_key = "id"

  belongs_to :sidfmd_queue
  has_many :sidfmd_queue_win_details, :dependent => :destroy
  has_many :sidfmd_queue_win_apps, :dependent => :destroy

  ### FIXME
  OSMAPS = {

    :'10.0' => {
      :'20348' => {
        :name => 'Microsoft Windows Server 2022'.freeze,
        :product_id => 14, :version_id => 12264,
      }.freeze,
      :'19043' => {
        :name => 'Microsoft Windows 10'.freeze,
        :product_id => 14, :version_id => 8169,
      }.freeze,
      :'19042' => {
        :name => 'Microsoft Windows 10'.freeze,
        :product_id => 14, :version_id => 8169,
      }.freeze,
      :'19041' => {
        :name => 'Microsoft Windows 10'.freeze,
        :product_id => 14, :version_id => 8169,
      }.freeze,
      :'18363' => {
        :name => 'Microsoft Windows 10'.freeze,
        :product_id => 14, :version_id => 8169,
      }.freeze,
      :'18362' => {
        :name => 'Microsoft Windows Server version 1903'.freeze,
        :product_id => 14, :version_id => 10606,
      }.freeze,
      :'17763' => {
        :name => 'Microsoft Windows Server 2019'.freeze,
        :product_id => 14, :version_id => 10155,
      }.freeze,
      :'17134' => {
        :name => 'Microsoft Windows Server version 1803'.freeze,
        :product_id => 14, :version_id => 9893,
      }.freeze,
      :'16299' => {
        :name => 'Microsoft Windows Server version 1709'.freeze,
        :product_id => 14, :version_id => 9589,
      }.freeze,
      :'15063' => {
        :name => 'Microsoft Windows 10 Creators Update'.freeze,
        :product_id => 14, :version_id => 8169,
      }.freeze,
      :'14393' => {
        :name => 'Microsoft Windows Server 2016'.freeze,
        :product_id => 14, :version_id => 8994,
      }.freeze,
      :'10240' => {
        :name => 'Microsoft Windows 10'.freeze,
        :product_id => 14, :version_id => 8169,
      }.freeze,
      :'22000' => {
        :name => 'Microsoft Windows 11'.freeze,
        :product_id => 14, :version_id => 12365,
      }.freeze,
    }.freeze,
    :'6.3' => {
      :'9600' => {
        :name => 'Microsoft Windows Server 2012 R2'.freeze,
        :product_id => 14, :version_id => 6482,
      }.freeze,
    }.freeze,
    :'6.2' => {
      :'9200' => {
        :name => 'Microsoft Windows Server 2012'.freeze,
        :product_id => 14, :version_id => 6482,
      }.freeze,
    }.freeze,
    :'6.1' => {
      :'7602' => {
        :name => 'Microsoft Windows Server 2008 R2 SP1'.freeze,
        :product_id => 14, :version_id => 4763,
      }.freeze,
      :'7601' => {
        :name => 'Microsoft Windows Server 2008 R2 SP1'.freeze,
        :product_id => 14, :version_id => 4763,
      }.freeze,
      :'7600' => {
        :name => 'Microsoft Windows Server 2008 R2'.freeze,
        :product_id => 14, :version_id => 4763,
      }.freeze,
    }.freeze,
    :'6.0' => {
      :'6002' => {
        :name => 'Microsoft Windows Server 2008 SP2'.freeze,
        :product_id => 14, :version_id => 4108,
      }.freeze,
      :'6001' => {
        :name => 'Microsoft Windows Server 2008 SP1'.freeze,
        :product_id => 14, :version_id => 4108,
      }.freeze,
      :'6000' => {
        :name => 'Microsoft Windows Vista',
        :product_id => 14, :version_id => 231,
      }.freeze,
    }.freeze,
    '5.2'.freeze => {
      :'3790' => {
        :name => 'Microsoft Windows Server 2003',
        :product_id => 14, :version_id => 230,
      }.freeze,
    }.freeze,

  }.freeze

  UNKNOWN = { :name => 'サポート外'.freeze, :product_id => 14, :version_id => 0 }.freeze

  def os_info
    ret = UNKNOWN
    if os then
      os_parts = os.split(/\./)
      if os_parts.size >= 2 then
        os_version = os_parts[0] + '.' + os_parts[1]
        os_build = (os_parts.size == 3 ? os_parts[2] : '')
        l1 = OSMAPS[os_version.to_sym]
        if l1 then
          l2 = l1[os_build.to_sym]
          if l2 then
            ret = l2
          end
        end
      end
    end
    # sidfmd_queue_wins.osname
    if osname then
      new_ret = { :name => osname, :product_id => ret[:product_id], :version_id => ret[:version_id]
 }
      ret = new_ret
      # osnameからosを認識できればそれを適用する。
      if /\AMicrosoft Windows 10/ =~ osname then
        ret[:version_id] = 8169
      elsif /\AMicrosoft Windows 11/ =~ osname then
        ret[:version_id] = 12365
      end
    end
    ret
  end

end
