#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmHostsControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  DAY_SECONDS = 24 * 60 * 60
  # ソート
  def sort_histories_content(content:, order:)
    separate_char_index = order.rindex('_')
    column = order.slice(0..separate_char_index - 1)
    order = order.slice(separate_char_index + 1..order.size - 1)

    case column
    when 'created_on' then
      content = content.order(%| journals.created_on #{order == 'asc' ? 'ASC' : 'DESC'} |)
    when 'login_name' then
      content = content.order(%| LOWER(users.login COLLATE "C") #{order == 'asc' ? 'ASC' : 'DESC'} |)
    when 'operation' then
      content = content.order(%|
        CASE journal_details.prop_key
          WHEN 'status_id' THEN 1
          WHEN 'due_date'THEN 2
          ELSE 3
        END
        #{order == 'asc' ? 'ASC' : 'DESC'} 
      |).order(%| journals.created_on #{order == 'asc' ? 'DESC' : 'ASC'} |)
    when 'hostname' then
      content = content.order(%|
        LOWER(
          (CASE WHEN sidfmd_hosts.ext_name IS NULL THEN sidfmd_hosts.hostname ELSE sidfmd_hosts.ext_name END) COLLATE "C"
        ) #{order == 'asc' ? 'ASC' : 'DESC'} 
      |)
    when 'vul_title' then
      content = content.order(%| abs(sidfmd_issues.html_id) #{order == 'asc' ? 'ASC' : 'DESC'}|)
                                                         .order(%| LOWER(sidfmd_issues.title COLLATE "C") #{order == 'asc' ? 'ASC' : 'DESC'} |)
    when 'status' then
      content = content.order(%| issues.status_id #{order == 'asc' ? 'ASC' : 'DESC'} |)
                                                         .order(%| journals.created_on #{order == 'asc' ? 'DESC' : 'ASC'} |)
    when 'due_date' then
      content = content.order(%| issues.due_date #{order == 'asc' ? 'ASC' : 'DESC'} |)
    end
    if column != 'created_on' then
      content = content.order(%| journals.created_on #{order == 'asc' ? 'DESC' : 'ASC'} |)
    end
    if column != 'login_name' then
      content = content.order(%| LOWER(users.login COLLATE "C") #{order == 'asc' ? 'ASC' : 'DESC'} |)
    end
    content = content.order(%| journals.id #{order == 'asc' ? 'ASC' : 'DESC'} |)
  end

  # フィルタの初期値
  def default_histories_filters
    filters = {}
    filters[:roles] = { :ids => [], :is_except => false }
    filters[:users] =  { :ids => [], :is_except => false, :is_except_anonymous => false }
    filters[:hosts] = { :ids => [], :is_except => false }
    filters[:categories] = { :ids => [], :is_except => false, :is_and => false }
    filters[:date] = '::'
    filters[:opes] = { :targets => [], :is_except => false }
    filters[:status] = '0'
    filters[:comment] = { :text => '', :is_except => false }
    filters[:vuls] = ''
    filters[:ctags] = { :ids => [] }
    filters
  end

  # フィルタ
  def filter_histories_content(content:, filters:, role_admin:, user_category_ids:)
    #権限検索
    if filters[:roles][:ids].any? then
      content =
        if filters[:roles][:is_except].to_s == 'true' then
          content.where(
            :users => { :id => Member.joins(:member_roles)
                                     .where.not(:member_roles => {:role_id => filters[:roles][:ids]})
                                     .select(:user_id)
                      }
          )
        else
          content.where(
            :users => { :id => Member.joins(:member_roles)
                                     .where(:member_roles => {:role_id => filters[:roles][:ids]})
                                     .select(:user_id)
                      }
          )
        end
    end

    #ユーザ検索
    if filters[:users][:is_except_anonymous].to_s == 'true' then
      content = content.where.not(:users => {:type => 'AnonymousUser'})
    end
    if filters[:users][:ids].any? then
      unless role_admin then
        sidfmadm_user = User.where(:id => filters[:users][:ids], :login => 'sidfmadm').first
        filters[:users][:ids] = filters[:users][:ids].map(&:to_i) & User.get_category_users(user_category_ids).pluck(:id)
        filters[:users][:ids] << sidfmadm_user.id if sidfmadm_user
      end
      content = if filters[:users][:is_except].to_s == 'true' then
                  content.where.not(:users => {:id => filters[:users][:ids]})
                else
                  content.where(:users => {:id => filters[:users][:ids]})
                end
    end

    #ホスト検索
    if filters[:hosts][:ids].any? then
      content = if filters[:hosts][:is_except].to_s == 'true' then
                  content.where.not(:sidfmd_hosts => {:id => filters[:hosts][:ids]})
                else
                  content.where(:sidfmd_hosts => {:id => filters[:hosts][:ids]})
                end
    end

    #カテゴリ検索
    if filters[:categories][:ids].any? then
      unless role_admin then
        filters[:categories][:ids] = (filters[:categories][:ids].map(&:to_i) & user_category_ids) + SidfmdCategory.where(:is_system => true).where(:id => filters[:categories][:ids]).pluck(:id)
      end
      target_host_ids = []
      if filters[:categories][:is_and].to_s == 'true' then
        SidfmdHost.where(
          :id => SidfmdCategoriesHost.where(:sidfmd_category_id => filters[:categories][:ids]).select(:sidfmd_host_id)
        ).where(:flg_delete => false)
        .each do |host|
          host_cat_ids = host.sidfmd_categories.pluck(:id)
          if (filters[:categories][:ids] - host_cat_ids).empty? then
            target_host_ids << host.id
          end
        end
      else
        target_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => filters[:categories][:ids])
                                              .select(:sidfmd_host_id)
      end

      content = if filters[:categories][:is_except].to_s == 'true' then
                  content.where.not(:sidfmd_hosts => { :id => target_host_ids })
                else
                  content.where(:sidfmd_hosts => {:id => target_host_ids})
                end
    end

    #期間指定検索
    if filters[:date] != '::' then
      if /^(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)$/ =~ filters[:date] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          filters[:date] = '::'
        else
          flg_rev = orig_flg_rev == 'rev' ? true : false
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          if flg_rev then
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                'journals.created_on <= ? OR journals.created_on >= ?', begin_date, end_date
              )
            elsif begin_date then
              begin_date = begin_date.to_time
              content = content.where(
                'journals.created_on <= ?', begin_date
              )
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                'journals.created_on >= ?', end_date
              )
            else
              filters[:date] = '::' # last resort
            end
          else
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                'journals.created_on BETWEEN ? AND ?', begin_date, end_date
              )
            elsif begin_date then
              begin_date = begin_date.to_time
              content = content.where(
                'journals.created_on >= ?', begin_date
              )
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                'journals.created_on <= ?', end_date
              )
            else
              filters[:date] = '::' # last resort
            end
          end
        end
      else
        filters[:date] = '::'
      end
    end

    #操作検索
    if filters[:opes][:targets].any? then
      targets = filters[:opes][:targets].select{|target| target == 'status_id' or target == 'due_date' } #フィルタ
      content = if filters[:opes][:is_except].to_s == 'true' then
                  content.where(%|
                    journal_details.prop_key IS NULL OR journal_details.prop_key NOT IN (?)
                  |, targets)
                else
                  content.where(:journal_details => { :prop_key => targets })
                end
    end

    # 対応状況
    if /\A(\d):(\d)\z/ =~ filters[:status] then
      status1 = Regexp.last_match(1).to_i
      status2 = Regexp.last_match(2).to_i
      #logger.info( [ status1, status2 ])
      content = content.where(
        :issues => { :status_id => [ status1, status2 ] }
      )
    elsif filters[:status] != '0' then
      content = content.where(
        :issues => { :status_id => filters[:status].to_i }
      )
    end

    #コメント検索
    if filters[:comment][:text].size > 0 or filters[:comment][:is_except].to_s == 'true' then
      content = if filters[:comment][:is_except].to_s == 'true' then
        content.where.not(%| journals.notes ILIKE ? |, "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:comment][:text])}%")
      else
        content.where(%| journals.notes ILIKE ? |, "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:comment][:text])}%")
      end
    end

    #脆弱性検索
    if filters[:vuls].size > 0 then
      if /\A\s*(\d+)\s*\z/ =~ filters[:vuls] then
        html_id = Regexp.last_match(1)
        content = content.where(%|
          sidfmd_issues.html_id = ? OR sidfmd_issues.title ILIKE ? 
        |, html_id.to_i, "%#{ActiveRecord::Base.send(:sanitize_sql_like, html_id.to_s.strip)}%")
      elsif /\A\s*KB(\d+)\s*\z/i =~ filters[:vuls] then
        kbid = -Regexp.last_match(1).to_i
        content = content.where(%|
          CAST(sidfmd_issues.html_id AS text) ILIKE ? OR sidfmd_issues.title ILIKE ?
        |, kbid.to_s, "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:vuls].strip)}%")
      else
        content = content.where(%|
          sidfmd_issues.title ILIKE ? OR sidfmd_issues.id IN (SELECT DISTINCT sidfmd_issue_id FROM sidfmd_issue_cves WHERE cve_id ILIKE ?)
        |,"%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:vuls].strip)}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:vuls].strip)}%")
      end
    end

    # カスタムタグ検索
    if filters[:ctags][:ids].any? then
      unless role_admin then
        filters[:ctags][:ids] = filters[:ctags][:ids] & SidfmdCtag.user_relatable_ctag_ids(role_admin, user_category_ids)
      end
      content = content.where(
        :sidfmd_issues => {
          :id => SidfmdIssueCtag.where(:sidfmd_ctag_id => filters[:ctags][:ids])
                                .select(:sidfmd_issue_id)
        }
      )
    end

    content
  end

end
