#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmLproductsControllerSupport
  include SidfmVmApiControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  def sort_lproducts_content(content:, order:)
    last_index = order.rindex('_')
    col = order.slice(0..last_index -1)
    order = SidfmControllerSupport.check_order(order.slice(last_index + 1..order.size - 1))
  
    if col == "priority" then
      content = content.order(:priority => order.to_sym)
    elsif col == "name" then
      content = content.order(%| LOWER(name COLLATE "C") #{order.upcase}|)
        .order(:priority => order.to_sym)
    elsif col == "status" then
      content = content.order(:flg_enable => order.to_sym == :asc ? :desc : :asc)
        .order(:priority => order.to_sym)
    elsif col == "is_support_end" then
      content = content.order(%|
        CASE WHEN eol_date < '#{Date.today}' THEN 0 ELSE 1 END #{order.upcase}
      |).order(:priority => order.to_sym)
    else #どれにも当てはまらない場合は、priority順にする。(普通の操作では起きない)
      content = content.order(:priority => order.to_sym)
    end
    content
  end

  def filter_lproducts_content(content:, filters:)
    if filters[:name] && filters[:name].size > 0 then
      content = content.where(
        'LOWER(name) LIKE ?',
        "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:name].downcase)}%"
      )
    end
    if filters[:version] && filters[:version].size > 0 then
      content = content.where(
        :id => SidfmdLversion.where(
          'LOWER(sidfmd_lversions.version) LIKE ?',
          "%#{ActiveRecord::Base.send(:sanitize_sql_like, filters[:version].downcase)}%"
        ).select(:sidfmd_lproduct_id)
      )
    end
    if filters[:flg_enable].to_s == 'true' || filters[:flg_enable].to_s == 'false' then
      content = content.where(:flg_enable => filters[:flg_enable])
    end
    if filters[:is_support_end].to_s == 'true' || filters[:is_support_end].to_s == 'false' then
      content =
        if filters[:is_support_end].to_s == 'true' then
          content.where("eol_date < ?", Date.today)
        else
          content.where("eol_date >= ? OR eol_date IS NULL", Date.today)
        end
    end
    content
  end

  def get_date(str_date)
    date = { :valid => true, :date => '' } 
    if str_date and str_date.size > 0 then
      begin
        date[:date] = Date.parse(str_date)
      rescue Exception => e
        date[:valid] = false
      end
    else
      date[:date] = nil
    end
    date
  end

  # 関連するルールを処理
  def process_related_rules(**opts)
    r = true
    param = {
      :role_admin => @sidfm_roles.include?(:admin),
      :action => opts[:action],
      :messages_necessary => false
    }
    case opts[:action]
    when 'delete' then
      param[:delete_lproduct_ids] = opts[:delete_lproduct_ids]
      param[:delete_lversion_ids] = []
    when 'modify' then
      param[:action] = 'delete'
      param[:delete_lproduct_ids] = []
      param[:delete_lversion_ids] = opts[:delete_lversion_ids]
    when 'import' then
      param[:action] = 'delete'
      param[:delete_lproduct_ids] = opts[:delete_lproduct_ids]
      param[:delete_lversion_ids] = opts[:delete_lversion_ids]
    end
    ret = SidfmdRule.lproducts_related_rule_info(param)
    SidfmdRuleCmLappSub.where(:id => ret[:delete_cm_lapp_ids]).destroy_all if ret[:delete_cm_lapp_ids].any?
    SidfmdRuleCond.where(:id => ret[:delete_cond_ids]).destroy_all if ret[:delete_cond_ids].any?
    SidfmdRuleAction.where(:id => ret[:delete_action_ids]).destroy_all if ret[:delete_action_ids].any?
    SidfmdRule.where(:id => ret[:delete_rule_ids]).destroy_all if ret[:delete_rule_ids].any?
    r
  end

  #新規登録
  def create_lproducts(
    prod_name:,
    priority:,
    lprod_eol_date:,
    flg_enable:,
    versions:
  )

    SidfmdLproduct.transaction do 
      begin 
        new_prod = SidfmdLproduct.new(
          flg_enable: flg_enable == 'enable' ? true : false,
          name: prod_name,
          priority: priority,
          eol_date: lprod_eol_date[:date]
        )
        f_versions = []
        is_pass = true
        versions.each do |version|
          version = version.symbolize_keys
          release_date = get_date(version[:release_date])
          support_date = get_date(version[:support_date])
          eol_date = get_date(version[:eol_date])
          [ release_date, support_date, eol_date ].each do |date|
            if !date[:valid] then
              is_pass = false
              break
            end
          end
          f_versions << SidfmdLversion.new(
            flg_enable: version[:flg_enable],
            version: version[:version],
            fullname: new_prod.name + ' ' + version[:version],
            priority: version[:priority],
            release_date: release_date[:date],
            support_date: support_date[:date],
            eol_date: eol_date[:date]
          )
        end
        if !is_pass then
          return create_process_result '不正な操作です。'
        end
        new_prod.sidfmd_lversions = f_versions
        if new_prod.save then
          #プロダクトの順位整列
          SidfmdLproduct.where('priority >= ?', priority).where.not(:id => new_prod.id).each do |p|
            p.update_column(:priority, p.priority + 1)
          end
        end
        Rails.cache.clear
        return create_process_result nil, { :id => new_prod.id }
      rescue Exception => e
        return create_process_result 'プロダクトの登録に失敗しました。'
      end
    end
  end

  def modify_lproducts(
    prod_name:,
    priority:,
    flg_enable:,
    lprod_eol_date:,
    versions:,
    prod_id:
  )
  
    begin 
      SidfmdLproduct.transaction do
        is_pass = true
        f_versions = []
        versions.each do |version|
          version = version.symbolize_keys
          release_date = get_date(version[:release_date])
          support_date = get_date(version[:support_date])
          eol_date = get_date(version[:eol_date])
          [ release_date, support_date, eol_date ].each do |date|
            if !date[:valid] then
              is_pass = false
              break
            end
          end
          version[:release_date] = release_date[:date]
          version[:support_date] = support_date[:date]
          version[:eol_date] = eol_date[:date]
          f_versions << version  
        end
        if !is_pass then
          return create_process_result '不正な操作です。'
        end
        versions = f_versions
        priority = priority.to_i
        prod = SidfmdLproduct.find_by_id(prod_id)
        if prod then
          current_versions = prod.sidfmd_lversions
          version_ids = versions.map{|v| v[:id]}
          del_versions = current_versions.where.not(:id => version_ids)
          del_version_ids = del_versions.pluck(:id)
          # 関連するルールの処理
          if del_version_ids.any? then
            process_related_rules(
              :delete_lversion_ids => del_version_ids,
              :action => 'modify'
            )
          end
          old_prod_name = prod.name
          old_priority = prod.priority
          prod.update_attributes(
            name: prod_name,
            flg_enable:flg_enable == 'enable' ? true : false,
            priority: priority,
            eol_date: lprod_eol_date[:date]
          )
          #バージョン更新
          del_versions.destroy_all
          SidfmdLocalVulLapp.where(:sidfmd_lversion_id => del_version_ids).destroy_all
          SidfmdHostLapp.where(:sidfmd_lversion_id => del_version_ids).destroy_all
          versions.each do |v|
            target_version = current_versions.find_by_id(v[:id])
            if !target_version then
              SidfmdLversion.create(
                sidfmd_lproduct_id: prod.id, 
                flg_enable: v[:flg_enable],
                version: v[:version],
                fullname: prod.name + ' ' + v[:version],
                priority: v[:priority],
                release_date: v[:release_date],
                support_date: v[:support_date],
                eol_date: v[:eol_date]
              )
            else
              if old_prod_name != prod.name ||
                 target_version.flg_enable != v[:flg_enable] ||
                 target_version.version != v[:version] ||
                 target_version.priority != v[:priority] ||
                 target_version.release_date != v[:release_date] ||
                 target_version.support_date != v[:support_date] ||
                 target_version.eol_date != v[:eol_date]
              then 
                target_version.update_attributes(
                  flg_enable: v[:flg_enable],
                  version: v[:version],
                  fullname: prod.name + ' ' + v[:version],
                  priority: v[:priority],
                  release_date: v[:release_date],
                  support_date: v[:support_date],
                  eol_date: v[:eol_date]
                )
              end  
            end 
          end 
          #プロダクトの順位整列
          if old_priority != prod.priority then
            if prod.priority > old_priority then
              SidfmdLproduct.where('priority > ?', old_priority)
                            .where('priority <= ?', prod.priority)
                            .where.not(:id => prod.id)
                            .each do |p|
                p.update_attribute(:priority, p.priority - 1)
              end
            else
              SidfmdLproduct.where('priority < ?', old_priority)
                            .where('priority >= ?', prod.priority)
                            .where.not(:id => prod.id)
                            .each do |p|
                p.update_attribute(:priority, p.priority + 1)
              end
            end
          end
        end
      end
      Rails.cache.clear
      return create_process_result nil
    rescue Exception => e
      return create_process_result 'バージョンの更新に失敗しました。'
    end
  end

  def delete_lproducts(delete_id:)
    begin
      SidfmdLproduct.transaction do
        prod = SidfmdLproduct.find_by_id(delete_id)
        if prod then
          # 関連するルールの処理
          process_related_rules(
            :delete_lproduct_ids => [ delete_id ],
            :action => 'delete'
          )
          prod_priority = prod.priority
          if SidfmdLversion.where(:sidfmd_lproduct_id => delete_id).destroy_all && prod.destroy then
            #プロダクトの順位整列
            SidfmdLproduct.where('priority > ?', prod_priority).each do |p|
              p.update_column(:priority, p.priority - 1)
            end
            #ロカール脆弱性の関連削除
            SidfmdLocalVulLapp.where(:sidfmd_lproduct_id => delete_id).destroy_all
            #ホストの関連削除
            SidfmdHostLapp.where(:sidfmd_lproduct_id => delete_id).destroy_all
          end
        end
      end
      Rails.cache.clear
      return create_process_result nil
    rescue Exception => e
      return create_process_result 'プロダクトの削除に失敗しました。'
    end
  end
end
