#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmHostsControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  DAY_SECONDS = 24 * 60 * 60
  # ソート
  def sort_hosts_content(content:, order:, role_admin:, user_category_ids:, prior_category:, index:)
    case order
    when 'warning_asc' then
      content = content.order("warning_#{index.to_s} DESC").order('LOWER(display_name COLLATE "C")')
    when 'warning_desc' then
      content = content.order("warning_#{index.to_s}").order('LOWER(display_name COLLATE "C") DESC')
    when 'statuses_asc' then
      content = content.order(:min_status_id).order('LOWER(display_name COLLATE "C")')
    when 'statuses_desc' then
      content = content.order(:min_status_id => :desc).order('LOWER(display_name COLLATE "C") DESC')
    when 'maxsri_asc' then
      content = content.order(:max_sri => :desc).order('LOWER(display_name COLLATE "C")')
    when 'maxsri_desc' then
      content = content.order(:max_sri).order('LOWER(display_name COLLATE "C") DESC')
    when 'maxcvss_asc', 'maxcvss_desc' then
      content = content.order(%|
                  CASE 
                  WHEN max_cvss_score IS NULL THEN
                    0.0
                  ELSE
                    max_cvss_score
                  END
                  #{order == 'maxcvss_asc' ? 'DESC': 'ASC'}
                |)
                .order(%| LOWER(display_name COLLATE "C")  #{order == 'maxcvss_asc' ? 'ASC': 'DESC'} |)
    when 'area_asc' then
      content = content.order("CASE min_status_id WHEN #{SidfmdIssue::STATUS_NOTAFFECT} THEN 2 ELSE 1 END")
                       .order(:max_area_id => :desc)
                       .order('LOWER(display_name COLLATE "C")')
    when 'area_desc' then
      content = content.order("CASE min_status_id WHEN #{SidfmdIssue::STATUS_NOTAFFECT} THEN 1 ELSE 2 END")
                       .order(:max_area_id)
                       .order('LOWER(display_name COLLATE "C") DESC')
    when 'exploitable_asc', 'exploitable_desc' then
      content = content.order(%|
        CASE
        WHEN max_cvss_label = 'notreg' THEN
          3
        WHEN exploitable IS FALSE THEN
          2
        ELSE
          1
        END
        #{order == 'exploitable_asc' ? 'ASC' : 'DESC'}
      |).order(%| LOWER(display_name COLLATE "C") #{order == 'exploitable_asc' ? 'ASC' : 'DESC'}|)
    when 'hostname_asc' then
      content = content.order('LOWER(display_name COLLATE "C")')
    when 'hostname_desc' then
      content = content.order('LOWER(display_name COLLATE "C") DESC')
    when 'categories_asc', 'categories_desc' then
      if prior_category == 'custom' then
        content = content.joins(%|
          LEFT OUTER JOIN(
            SELECT 
              sidfmd_host_view_id AS order_custom_category_host_view_id,
              MIN(LOWER(name COLLATE "C")) AS order_custom_category_name
            FROM
              sidfmd_hv_categories
            #{(role_admin || !user_category_ids.any?) ? '' : "WHERE sidfmd_category_id IN (SELECT unnest(ARRAY[#{user_category_ids.join(', ')}]))"}
            GROUP BY sidfmd_host_view_id
          ) AS order_custom_min_name_category
          ON sidfmd_host_views.id = order_custom_min_name_category.order_custom_category_host_view_id
        |)
        .order(%|
          CASE WHEN order_custom_category_name IS NULL THEN
            system_category_id
          ELSE
            0
          END
          #{order == 'categories_asc' ? 'ASC' : 'DESC'}
        |)
        .order(%|
          LOWER(order_custom_category_name COLLATE "C")
          #{order == 'categories_asc' ? 'ASC' : 'DESC'}
        |)
        .order(%| LOWER(display_name COLLATE "C") #{order == 'categories_asc' ? 'ASC' : 'DESC'} |)
      else
        content = content.order(%| system_category_id #{order == 'categories_asc' ? 'ASC' : 'DESC'} |)
                         .order(%| LOWER(display_name COLLATE "C") #{order == 'categories_asc' ? 'ASC' : 'DESC'} |)
      end
    when 'os_asc' then
      content = content.order(:osname).order('LOWER(display_name COLLATE "C")')
    when 'os_desc' then
      content = content.order(:osname => :desc).order('LOWER(display_name COLLATE "C") DESC')
    when 'duedates_asc' then
      content = content.order('ROUND(CAST(duedates_percent as numeric), 1) ASC')
                       .order('LOWER(display_name COLLATE "C")')
    when 'duedates_desc' then
      content = content.order('ROUND(CAST(duedates_percent as numeric), 1) DESC')
                       .order('LOWER(display_name COLLATE "C") DESC')
    when 'finished_asc' then
      content = content.order("CASE min_status_id WHEN #{SidfmdIssue::STATUS_NOTAFFECT} THEN 1 ELSE 0 END")
                       .order('ROUND(CAST(fixed_percent as numeric), 1) ASC')
                       .order('CASE fixed_percent WHEN NULL THEN 1 ELSE 0 END')
                       .order('LOWER(display_name COLLATE "C")')
    when 'finished_desc' then
      content = content.order("CASE min_status_id WHEN #{SidfmdIssue::STATUS_NOTAFFECT} THEN 0 ELSE 1 END")
                       .order('ROUND(CAST(fixed_percent as numeric), 1) DESC')
                       .order('CASE fixed_percent WHEN NULL THEN 0 ELSE 1 END')
                       .order('LOWER(display_name COLLATE "C") DESC')
    end

    content
  end

  #ホストのOS一覧
  def hosts_dist_versions(content, is_view)
    dist_versions = if is_view then
                       content.where('LENGTH(osname) >= 1').order(:osname).distinct.pluck(:osname)
                     else
                       content.collect {|x|
                         (x[:os] and x[:os][:osname] and x[:os][:osname].size > 0) ? x[:os][:osname] : nil
                       }.compact.uniq.sort
                     end
  end

  # フィルタの初期値
  def default_hosts_filters(dist_versions)
    filters = {}
    filters[:warning] = { :values => [ 'warning', 'others', 'all' ].map(&:freeze).freeze, :default => 'all'.freeze }
    filters[:status] = {
      :values => (SidfmdIssue::STATUS_ALL.collect {|x| x.to_s } +
                  [ SidfmdIssue::STATUS_UNFINISHED.collect {|x| x.to_s }.join(':') ] +
                  [ SidfmdIssue::STATUS_FINISHED.collect {|x| x.to_s }.join(':') ] +
                  [ '0' ]).map(&:freeze).freeze, :default => '0'.freeze }
    filters[:sris] = {
      :values => SidfmdIssueSri::VALUES.map{|v| [v.to_s, v]}.to_h,
      :default => [].freeze
    }
    filters[:cvsses] = {
      :values => SidfmdIssueCve::DISPLAY_SORTED_LABELS.map{|v| [v.to_s, v]}.to_h,
      :default => [].freeze
    }
    filters[:area] = { :values => (SidfmdArea::VALUES.collect {|x| x.to_s } + [ '' ]).map(&:freeze).freeze, :default => ''.freeze }
    filters[:exploitable] = { :values => [ 'true', '' ].map(&:freeze).freeze, :default => ''.freeze }
    filters[:categories] = {
      :values => nil,
      :type_source => { #type_sourceはdefaultでは型が不明な場合に使う。各値には期待されるデータと同じ型のデータを入れる
        :ids => [0].freeze,
        :is_and => false,
        :is_except => false
      },
      :default => {
        :ids => [].freeze,
        :is_and => false,
        :is_except => false
      }
    }
    filters[:os] = { :values => dist_versions + [ ''.freeze ], :default => '' }

    filters[:due_date] = { :values => nil, :default => '::' }

    filters[:keyword] = { :values => nil, :default => '' }
    filters[:date] = { :values => nil, :default => '::' }
    filters[:products] = {
      :values => nil,
      :type_source => {
        :product_all_version_ids => [0],
        :product_ids => [0],
        :version_ids => [0],
        :lproduct_all_version_ids => [0],
        :lproduct_ids => [0],
        :lversion_ids => [0]
      },
      :default => {
        :product_all_version_ids => [],
        :product_ids => [],
        :version_ids => [],
        :lproduct_all_version_ids => [],
        :lproduct_ids => [],
        :lversion_ids => []
      }
    }
    filters[:eol_products] = { :values => [ 'true', '' ].map(&:freeze).freeze, :default => ''.freeze }
    filters
  end

  # フィルタ適用
  def filter_hosts_content(content:, filters:, role_admin:, user_category_ids:, user_host_ids:, prior_category:, index:)
    #プロダクト検索
    if filters[:products].values.select{|v| v.size > 0 }.any? then
      prods_related_host_ids = []
      prod_info = filters[:products].symbolize_keys
      if prod_info[:product_all_version_ids] and prod_info[:product_all_version_ids].any? then
        #全バージョン
        prods_related_host_ids |= SidfmdHostApp.where(:product_id => prod_info[:product_all_version_ids]).pluck(:sidfmd_host_id)
      end

      if prod_info[:product_ids] and prod_info[:product_ids].any? then
        #「全て」のバージョン
        prods_related_host_ids |= SidfmdHostApp.where(:product_id => prod_info[:product_ids])
                                               .where('version_id = 0 OR version_id IS NULL')#バージョンは該当させない
                                               .pluck(:sidfmd_host_id)
      end

      if prod_info[:version_ids] and prod_info[:version_ids].any? then
        #versionが検索対象
        prods_related_host_ids |= SidfmdHostApp.where(:version_id => prod_info[:version_ids]).pluck(:sidfmd_host_id)
      end

      if prod_info[:lproduct_all_version_ids] and prod_info[:lproduct_all_version_ids].any? then
        #全バージョン
        prods_related_host_ids |= SidfmdHostLapp.where(:sidfmd_lproduct_id => prod_info[:lproduct_all_version_ids]).pluck(:sidfmd_host_id)
      end

      if prod_info[:lproduct_ids] and prod_info[:lproduct_ids].any? then
        #「全て」のバージョン
        prods_related_host_ids |= SidfmdHostLapp.where(:sidfmd_lproduct_id => prod_info[:lproduct_ids])
                                                .where(:sidfmd_lversion_id => 0)#バージョンは該当させない
                                                .pluck(:sidfmd_host_id)
      end

      if prod_info[:lversion_ids] and prod_info[:lversion_ids].any? then
        #lversionが検索対象
        prods_related_host_ids |= SidfmdHostLapp.where(:sidfmd_lversion_id => prod_info[:lversion_ids]).pluck(:sidfmd_host_id)
      end

      if prods_related_host_ids.any? then
        content = content.where(:sidfmd_host_id => prods_related_host_ids)
      else
        content = content.where(:sidfmd_host_id => 0)
      end
    end

    #eol検索
    if filters[:eol_products].to_s == 'true' then
      content = content.where(
        "sidfmd_host_id IN (?) OR sidfmd_host_id IN (?) OR sidfmd_host_id IN (?) OR sidfmd_host_id IN (?)",
        SidfmdHostApp.where(:product_id => SidfmdProduct.support_end_products.select(:pid))
          .select(:sidfmd_host_id),
        SidfmdHostApp.where(:version_id => SidfmdVersion.support_end_versions.select(:vid))
          .select(:sidfmd_host_id),
        SidfmdHostLapp.where(:sidfmd_lproduct_id => SidfmdLproduct.support_end_products.select(:id))
           .select(:sidfmd_host_id),
        SidfmdHostLapp.where(:sidfmd_lversion_id => SidfmdLversion.support_end_versions.select(:id))
           .select(:sidfmd_host_id)
      )
    end

    # 要注意
    if filters[:warning] == 'warning' then
      if index == :sri then
        content = content.where(:warning_sri => true)
      else
        content = content.where(:warning_cvss => true)
      end
    elsif filters[:warning] == 'others' then
      if index == :sri then
        content = content.where.not(:warning_sri => true)
      else
        content = content.where.not(:warning_cvss => true)
      end
    end

    # 対応状況
    # min_status_idで判断
    if filters[:status] != '0' then
      if /\A(\d):(\d)\z/ =~ filters[:status] then
        status1 = Regexp.last_match(1).to_i
        status2 = Regexp.last_match(2).to_i
        content = content.where('min_status_id IN (?)', [status1, status2])
      else
        content = content.where(:min_status_id => filters[:status].to_i)
      end
    end

    # 最大SRI
    if filters[:sris].any? then
      content = content.where(:max_sri => filters[:sris])
    end

    # 最大CVSS
    if filters[:cvsses].any? then
      content = content.where(:max_cvss_label => filters[:cvsses].map{|cvss| cvss.to_s})
    end

    # 影響範囲
    if filters[:area] != '' then
      content = content.where(:id => SidfmdHvScsid.where('status_id > 0').select(:sidfmd_host_view_id))
                       .where(:kb => false)
                       .where(:max_area_id => filters[:area].to_i)
    end

    # 攻撃コード
    if filters[:exploitable].to_s == 'true' then
      content = content.where(:exploitable => true)
    end

    #カテゴリ(複数)
    if filters[:categories][:ids].any? then
      unless role_admin then
        filters[:categories][:ids] = (filters[:categories][:ids].map(&:to_i) & user_category_ids) + SidfmdCategory.where(:is_system => true).where(:id => filters[:categories][:ids]).pluck(:id)
      end
      target_host_ids = []
      if filters[:categories][:is_and].to_s == 'true' then
        SidfmdHost.where(
          :id => SidfmdCategoriesHost.where(:sidfmd_category_id => filters[:categories][:ids]).select(:sidfmd_host_id)
        ).where(:flg_delete => false)
        .each do |host|
          host_cat_ids = host.sidfmd_categories.pluck(:id)
          if (filters[:categories][:ids] - host_cat_ids).empty? then
            target_host_ids << host.id
          end
        end
      else
        target_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => filters[:categories][:ids])
                                              .select(:sidfmd_host_id)
      end
      content = if filters[:categories][:is_except].to_s == 'true' then
                  content.where.not(:sidfmd_host_id => target_host_ids)
                else
                  content.where(:sidfmd_host_id => target_host_ids)
                end
    end

    # OS
    if filters[:os] != '' then
      content = content.where(:osname => filters[:os])
    end

    # 対策予定日設定検索
    if filters[:due_date] != '::' then
      if /\A(|notset):(|\d{4}\/\d{2}\/\d{2}):(|\d{4}\/\d{2}\/\d{2})\z/ =~ filters[:due_date] then
        notset = ('notset' == Regexp.last_match(1))
        begindate = Regexp.last_match(2)
        enddate = Regexp.last_match(3)
        if notset then
          #対策予定が設定されていない脆弱性をもつホスト
          content = content.where(:id => SidfmdHvDueDate.where(:d => nil).select(:sidfmd_host_view_id))
        else
          #対策予定が設定されている脆弱性をもつホスト
          begin
            if begindate.to_s.size > 0 then
              begindate = Date.parse(begindate)
            else
              begindate = nil
             end
          rescue Exception => e
            begindate = nil
          end
          begin
            if enddate.to_s.size > 0 then
              enddate = Date.parse(enddate)
            else
              enddate = nil
            end
          rescue Exception => e
            enddate = nil
          end
          due_date = nil
          if begindate and enddate then
            content = content.where(:id => SidfmdHvDueDate.where(:d => begindate..enddate).select(:sidfmd_host_view_id))
          elsif begindate then
            content = content.where(:id => SidfmdHvDueDate.where('d >= ?', begindate).select(:sidfmd_host_view_id))
          elsif enddate then
            content = content.where(:id => SidfmdHvDueDate.where('d <= ?', enddate).select(:sidfmd_host_view_id))
          end
        end
      end
    end

    # 日付指定検索
    if filters[:date] != '::' then
      if /^(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)$/ =~ filters[:date] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          filters[:date] = '::'
        else
          flg_rev = orig_flg_rev == 'rev' ? true : false
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          if flg_rev then
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(:id => SidfmdHvCreatedOn.where('dt <= ? OR dt >= ?', begin_date, end_date).select(:sidfmd_host_view_id))
            elsif begin_date then
              begin_date = begin_date.to_time
              content = content.where(:id => SidfmdHvCreatedOn.where('dt <= ?', begin_date).select(:sidfmd_host_view_id))
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(:id => SidfmdHvCreatedOn.where('dt >= ?', end_date).select(:sidfmd_host_view_id))
            else
              # last resort
              filters[:date] = '::'
            end
          else
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(:id => SidfmdHvCreatedOn.where(:dt => begin_date..end_date).select(:sidfmd_host_view_id))
            elsif begin_date then
              begin_date = begin_date.to_time
              content = content.where(:id => SidfmdHvCreatedOn.where('dt >= ?', begin_date).select(:sidfmd_host_view_id))
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(:id => SidfmdHvCreatedOn.where('dt <= ?', end_date).select(:sidfmd_host_view_id))
            else
              # last resort
              filters[:date] = '::'
            end
          end
        end
      else
        filters[:date] = '::'
      end
    end

    #ホスト名検索
    if filters[:keyword] != '' then
      kwd = filters[:keyword]
      content = content.where('hostname ILIKE ? OR display_name ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd)}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd)}%")
    end
 
    content
  end

end
