#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmLocalVulsControllerSupport
  include SidfmVmApiControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  CVSS_V3_VECTORS = SidfmdIssueCve.get_cvss_v3_vectors

  def create_regist_cvss_vectors(vectors_hash)
    vectors = []
    if vectors_hash then
      vectors_hash.each do |key, value|
        if !CVSS_V3_VECTORS[key.to_s] or !CVSS_V3_VECTORS[key.to_s]['values'][value.to_s] then
          vectors = [] #validation
          break
        end
        vectors << "#{key}:#{value}"
      end
    end
    vectors
  end

  #新規登録
  def create_local_vuls(
    vul_name:,
    area_id:,
    risk_id:,
    short_description:,
    description:,
    flg_enable:,
    affected_products:,
    affected_lproducts:,
    cve_ids:,
    s_cats_sri:,
    confirmation:,
    workaround:,
    related_info:,
    flg_exploitable:,
    sri_memo:
  )
    begin
      SidfmdLocalVul.transaction do 
        new_vul = SidfmdLocalVul.new(
                    flg_enable: flg_enable,
                    title: vul_name,
                    risk_id: risk_id,
                    area_id: area_id,
                    short_description: short_description,
                    description: description,
                    confirmation: confirmation,
                    workaround: workaround,
                    related: related_info,
                  )
        if new_vul.save then
          #更新履歴
          time_now = Time.now
          SidfmdLocalVulLog.create(
            sidfmd_local_vul_id: new_vul.id, 
            dt: time_now,
            mes: "#{time_now.strftime('%Y/%m/%d')} コンテンツ 登録"
          )
          #プロダクト
          affected_products.each do |prod_id, data_array|
            data_array.each do |data|
              SidfmdLocalVulApp.create(
                sidfmd_local_vul_id: new_vul.id,
                pid: prod_id.to_i,
                vid: data['vid'].to_i #バージョンの場合はvid > 0で、プロダクトの場合はvid == 0
              )
            end
          end
          #ローカルプロダクト
          affected_lproducts.each do |lprod_id, data_array|
            data_array.each do |data|
              SidfmdLocalVulLapp.create(
                sidfmd_local_vul_id: new_vul.id,
                sidfmd_lproduct_id: lprod_id.to_i,
                sidfmd_lversion_id: data['vid'].to_i #バージョンの場合はvid > 0で、プロダクトの場合はvid == 0
              )
            end
          end
          #CVE ID
          cve_ids.uniq{|cve_id| cve_id[:cve_id]}.each do |cve_id|
            unless cve_id[:cve_id] =~ /\ACVE\-[0-9]{4}\-[0-9]+\z/ then
              raise Exception #不正
            end
            regist_vectors = create_regist_cvss_vectors cve_id[:vectors]
            SidfmdLocalVulCve.create(
              sidfmd_local_vul_id: new_vul.id,
              cve_id: cve_id[:cve_id],
              c3_base_score: (regist_vectors.any? ? sprintf('%2.1f', cve_id[:cvss_score].to_f) : nil),
              c3_vector: ((regist_vectors.any? and cve_id[:cvss_score]) ? regist_vectors.join('/') : nil),
              c3_exploitable: cve_id[:exploitable].to_s == 'true'
            )  
          end
          #SRI
          s_cats_sri.each do |cat_id, risk_id|
            SidfmdLocalVulSri.create(
              sidfmd_local_vul_id: new_vul.id,
              category_id: cat_id,
              risk_id: risk_id,
              memo: sri_memo
            )
          end
        end
        Rails.cache.clear
        return create_process_result nil, { :id => new_vul.id }
      end
    rescue Exception => e 
      logger.info(e)
      return create_process_result '脆弱性の登録に失敗しました。'
    end
  end

  #更新
  def modify_local_vuls(
    vul_id:,
    vul_name:,
    area_id:,
    risk_id:,
    short_description:,
    description:,
    flg_enable:,
    affected_products:,
    affected_lproducts:,
    confirmation:,
    workaround:,
    related_info:,
    cve_ids:,
    s_cats_sri:,
    sri_memo:,
    comments:
  )
    begin
      SidfmdLocalVul.transaction do
        time_now = Time.now
        mod_vul = SidfmdLocalVul.find(vul_id)
        mod_vul.update_attributes(
          flg_enable: flg_enable,
          title: vul_name,
          risk_id: risk_id,
          area_id: area_id,
          short_description: short_description,
          description: description,
          confirmation: confirmation,
          workaround: workaround,
          related: related_info,
          updated_at: time_now
        )
        #更新履歴
        vul_comments = mod_vul.sidfmd_local_vul_logs
        comments.each do |cmt|
          cmt = cmt.symbolize_keys
          target_cmt = vul_comments.find_by(:id => cmt[:id])
          if target_cmt then
            if target_cmt.mes != cmt[:mes] then
              target_cmt.update_attributes(:dt => time_now, :mes => cmt[:mes])
            end
          else#id == 0は新しい更新履歴
            SidfmdLocalVulLog.create(
              sidfmd_local_vul_id: mod_vul.id,
              dt: time_now,
              mes: cmt[:mes]
            )
          end
        end
        #プロダクト
        mod_vul.sidfmd_local_vul_apps.destroy_all
        affected_products.each do |prod_id, data_array|
          data_array.each do |data|
            SidfmdLocalVulApp.create(
              sidfmd_local_vul_id: mod_vul.id,
              pid: prod_id.to_i,
              vid: data['vid'].to_i #バージョンの場合はvid > 0で、プロダクトの場合はvid == 0
            )
          end
        end
        #ローカルプロダクト
        mod_vul.sidfmd_local_vul_lapps.destroy_all
        affected_lproducts.each do |lprod_id, data_array|
          data_array.each do |data|
            SidfmdLocalVulLapp.create(
              sidfmd_local_vul_id: mod_vul.id,
              sidfmd_lproduct_id: lprod_id.to_i,
              sidfmd_lversion_id: data['vid'].to_i #バージョンの場合はvid > 0で、プロダクトの場合はvid == 0
            )
          end
        end
        #CVE ID
        mod_vul.sidfmd_local_vul_cves.destroy_all
        cve_ids.uniq{|cve_id| cve_id[:cve_id]}.each do |cve_id|
          unless cve_id[:cve_id] =~ /\ACVE\-[0-9]{4}\-[0-9]+\z/ then
            raise Exception #不正
          end
          regist_vectors = create_regist_cvss_vectors cve_id[:vectors]
          SidfmdLocalVulCve.create(
            sidfmd_local_vul_id: mod_vul.id,
            cve_id: cve_id[:cve_id],
            c3_base_score: (regist_vectors.any? ? sprintf('%2.1f', cve_id[:cvss_score].to_f) : nil),
            c3_vector: ((regist_vectors.any? and cve_id[:cvss_score]) ? regist_vectors.join('/') : nil),
            c3_exploitable: cve_id[:exploitable].to_s == 'true'
          )
        end
        #SRI
        mod_vul.sidfmd_local_vul_sris.destroy_all
        s_cats_sri.each do |cat_id, risk_id|
          SidfmdLocalVulSri.create(
            sidfmd_local_vul_id: mod_vul.id,
            category_id: cat_id,
            risk_id: risk_id,
            memo: sri_memo
          )
        end
      end
      Rails.cache.clear
      return create_process_result nil
    rescue Exception => e
      logger.info(e)
      return create_process_result e.message
    end
  end

  #削除
  def delete_local_vuls(delete_id:)
    err_message = ''
    host_ids = []
    html_id = 0
    SidfmdLocalVul.transaction do 
      begin 
        vul = SidfmdLocalVul.find(delete_id)
        html_id = vul.html_id
        #view関連のホストidを取得
        host_ids = SidfmdVulView.relate_hostids([ html_id ]) 
        vul.destroy
        issue_ids = SidfmdIssue.where(:html_id => html_id).pluck(:issue_id)
        if issue_ids.any? then
          SidfmdIssueCtag.where(:sidfmd_issue_id => issue_ids).delete_all
          Issue.where(:id => issue_ids).destroy_all
        end
      rescue Exception => e 
        logger.info(e)
        return create_process_result '脆弱性の削除に失敗しました。'
      end
    end
    # r = {
    #   :err_message => err_message,
    #   :host_ids => host_ids,
    #   :html_id => html_id  
    # }
    Rails.cache.clear
    return create_process_result nil # return r
  end
end
